@@ -27,9 +27,10 @@ class YOLOXHeadModule(BaseModule):
     Args:
         num_classes (int): Number of categories excluding the background
             category.
-        in_channels (int): Number of channels in the input feature map.
+        in_channels (Union[int, Sequence]): Number of channels in the input
+            feature map.
         widen_factor (float): Width multiplier, multiply number of
-            channels in each layer by this amount. Default: 1.0.
+            channels in each layer by this amount. Defaults to 1.0.
         num_base_priors (int): The number of priors (points) at a point
             on the feature grid
         stacked_convs (int): Number of stacking convs of the head.
@@ -186,7 +187,7 @@ class YOLOXHead(YOLOv5Head):
     """YOLOXHead head used in `YOLOX <https://arxiv.org/abs/2107.08430>`_.
 
     Args:
-        head_module(nn.Module): Base module used for YOLOXHead
+        head_module(ConfigType): Base module used for YOLOXHead
         prior_generator: Points generator feature maps in
             2D points-based detectors.
         loss_cls (:obj:`ConfigDict` or dict): Config of classification loss.
@@ -203,7 +204,7 @@ class YOLOXHead(YOLOv5Head):
     """
 
     def __init__(self,
-                 head_module: nn.Module,
+                 head_module: ConfigType,
                  prior_generator: ConfigType = dict(
                      type='mmdet.MlvlPointGenerator',
                      offset=0,
